#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_sequence.interfaces import ISequentialIntIds
from pyams_skin.layer import IPyAMSLayer
from pyams_utils.interfaces import VIEW_SYSTEM_PERMISSION, MANAGE_SYSTEM_PERMISSION
from pyams_utils.interfaces.intids import IIndexLength
from z3c.form.interfaces import DISPLAY_MODE, IDataExtractedEvent

# import packages
from pyams_form.form import AJAXEditForm
from pyams_pagelet.pagelet import pagelet_config
from pyams_zmi.form import AdminDialogEditForm
from pyramid.events import subscriber
from pyramid.view import view_config
from z3c.form import field
from zope.interface import Invalid

from pyams_sequence import _


@pagelet_config(name='properties.html', context=ISequentialIntIds, layer=IPyAMSLayer, permission=VIEW_SYSTEM_PERMISSION)
class SequentialIntIdsPropertiesEditForm(AdminDialogEditForm):
    """Sequential IDs properties edit form"""

    @property
    def title(self):
        return self.context.__name__

    legend = _("Display sequence properties")
    fields = field.Fields(IIndexLength) + field.Fields(ISequentialIntIds)

    ajax_handler = 'properties.json'
    edit_permission = MANAGE_SYSTEM_PERMISSION

    def updateWidgets(self, prefix=None):
        super(SequentialIntIdsPropertiesEditForm, self).updateWidgets(prefix)
        if 'count' in self.widgets:
            self.widgets['count'].mode = DISPLAY_MODE


@subscriber(IDataExtractedEvent, form_selector=SequentialIntIdsPropertiesEditForm)
def handle_sequence_data_extraction(event):
    """Handle sequence properties data extraction"""
    last_oid = event.data.get('last_oid')
    if last_oid < event.form.context.last_oid:
        event.form.widgets.errors += (Invalid(_("Can't set last OID to value lower than current one!")),)


@view_config(name='properties.json', context=ISequentialIntIds, request_type=IPyAMSLayer,
             permission=MANAGE_SYSTEM_PERMISSION, renderer='json', xhr=True)
class SequentialIntIdsPropertiesAJAXEditForm(AJAXEditForm, SequentialIntIdsPropertiesEditForm):
    """Sequential IDs properties edit form, JSON renderer"""
